@@ -98,10 +98,10 @@ class PasaffeWindow(Window):
         else:
             self.database = database
 
-        self.settings = Gio.Settings("net.launchpad.pasaffe")
+        self.settings = Gio.Settings.new("net.launchpad.pasaffe")
         self.settings.connect('changed', self.on_preferences_changed)
 
-        self.state = Gio.Settings("net.launchpad.pasaffe.state")
+        self.state = Gio.Settings.new("net.launchpad.pasaffe.state")
 
         # If database doesn't exists, make a new one
         if os.path.exists(self.database):
@@ -488,67 +488,133 @@ class PasaffeWindow(Window):
        return item, None
 
     def display_data(self, entry_uuid, show_secrets=False):
-        title = self.passfile.records[entry_uuid].get(3)
+        ttt = self.get_texttagtable()
+        data_buffer = Gtk.TextBuffer.new(ttt)
 
-        url = None
+        # title
+        data_buffer.insert(data_buffer.get_start_iter(), "\n")
+        data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                     self.passfile.records[entry_uuid].get(3),
+                                     ttt.lookup('title'))
+        data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
+        # url
         if 13 in self.passfile.records[entry_uuid]:
-            url = "%s\n\n" % self.passfile.records[entry_uuid].get(13)
+            data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                         _("URL:") + "\n",
+                                         ttt.lookup('section'))
+            data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                         self.passfile.records[entry_uuid].get(13),
+                                         ttt.lookup('url'))
+            data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
 
         contents = ''
         if show_secrets == False and \
            self.settings.get_boolean('only-passwords-are-secret') == False and \
            self.settings.get_boolean('visible-secrets') == False:
-            contents += _("Secrets are currently hidden.")
+            data_buffer.insert(data_buffer.get_end_iter(),
+                               _("Secrets are currently hidden."))
         else:
-            if 5 in self.passfile.records[entry_uuid]:
-                contents += "%s\n\n" % self.passfile.records[entry_uuid].get(5)
-            contents += _("Username: %s\n") % self.passfile.records[entry_uuid].get(4)
+            # username
+            data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                         _("Username:") + "\n",
+                                         ttt.lookup('section'))
+            data_buffer.insert(data_buffer.get_end_iter(),
+                               self.passfile.records[entry_uuid].get(4))
+            data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
+            # password
+            data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                         _("Password:") + "\n",
+                                         ttt.lookup('section'))
+
             if show_secrets == True or self.settings.get_boolean('visible-secrets') == True:
-                contents += _("Password: %s\n\n") % self.passfile.records[entry_uuid].get(6)
+                data_buffer.insert(data_buffer.get_end_iter(),
+                                   self.passfile.records[entry_uuid].get(6))
             else:
-                contents += _("Password: *****\n\n")
-            if 12 in self.passfile.records[entry_uuid]:
-                last_updated = time.strftime("%a, %d %b %Y %H:%M:%S",
-                               time.localtime(struct.unpack("<I",
-                               self.passfile.records[entry_uuid][12])[0]))
-                contents += _("Last updated: %s\n") % last_updated
-            if 8 in self.passfile.records[entry_uuid]:
-                pass_updated = time.strftime("%a, %d %b %Y %H:%M:%S",
-                               time.localtime(struct.unpack("<I",
-                                   self.passfile.records[entry_uuid][8])[0]))
-                contents += _("Password updated: %s\n") % pass_updated
-        self.fill_display(title, url, contents)
+                data_buffer.insert(data_buffer.get_end_iter(), '*****')
+            data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
+            # notes
+            if 5 in self.passfile.records[entry_uuid]:
+                data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                             _("Notes:") + "\n",
+                                             ttt.lookup('section'))
+                data_buffer.insert(data_buffer.get_end_iter(),
+                                   self.passfile.records[entry_uuid].get(5))
+                data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
+            # modification time
+            last_updated = self.passfile.get_modification_time(entry_uuid)
+            if last_updated != None:
+                data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                             _("Last updated:") + "\n",
+                                             ttt.lookup('section'))
+                data_buffer.insert(data_buffer.get_end_iter(),
+                                   last_updated)
+                data_buffer.insert(data_buffer.get_end_iter(), "\n")
+
+            # password updated time
+            pass_updated = self.passfile.get_password_time(entry_uuid)
+            if pass_updated != None:
+                data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                             _("Password updated:") + "\n",
+                                             ttt.lookup('section'))
+                data_buffer.insert(data_buffer.get_end_iter(),
+                                   pass_updated)
+                data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
+        self.ui.textview1.set_buffer(data_buffer)
 
     def display_welcome(self):
-        self.fill_display(_("Welcome to Pasaffe!"), None,
-                          _("Pasaffe is an easy to use\npassword manager for Gnome."))
+        ttt = self.get_texttagtable()
+        data_buffer = Gtk.TextBuffer.new(ttt)
+
+        data_buffer.insert(data_buffer.get_start_iter(), "\n")
+        data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                     _("Welcome to Pasaffe!"),
+                                     ttt.lookup('title'))
+        data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
+        data_buffer.insert(data_buffer.get_end_iter(),
+                           _("Pasaffe is an easy to use\npassword manager."))
+
+        self.ui.textview1.set_buffer(data_buffer)
 
     def display_folder(self, folder_name):
-        self.fill_display(folder_name, None,
-                          _("This is a folder."))
+        ttt = self.get_texttagtable()
+        data_buffer = Gtk.TextBuffer.new(ttt)
 
-    def fill_display(self, title, url, contents):
+        data_buffer.insert(data_buffer.get_start_iter(), "\n")
+        data_buffer.insert_with_tags(data_buffer.get_end_iter(),
+                                     folder_name,
+                                     ttt.lookup('title'))
+        data_buffer.insert(data_buffer.get_end_iter(), "\n\n")
+
+        data_buffer.insert(data_buffer.get_end_iter(),
+                           _("This is a folder."))
+
+        self.ui.textview1.set_buffer(data_buffer)
+
+    def get_texttagtable(self):
         texttagtable = Gtk.TextTagTable()
-        texttag_big = Gtk.TextTag.new("big")
+        texttag_big = Gtk.TextTag.new("title")
         texttag_big.set_property("weight", Pango.Weight.BOLD)
-        texttag_big.set_property("size", 12 * Pango.SCALE)
+        texttag_big.set_property("size", 14 * Pango.SCALE)
         texttagtable.add(texttag_big)
 
+        texttag_section = Gtk.TextTag.new("section")
+        texttag_section.set_property("weight", Pango.Weight.BOLD)
+        texttagtable.add(texttag_section)
+
         texttag_url = Gtk.TextTag.new("url")
         texttag_url.set_property("foreground", "blue")
         texttag_url.set_property("underline", Pango.Underline.SINGLE)
         texttag_url.connect("event", self.url_event_handler)
         texttagtable.add(texttag_url)
 
-        data_buffer = Gtk.TextBuffer.new(texttagtable)
-        data_buffer.insert_with_tags(data_buffer.get_start_iter(), "\n" + title + "\n\n", texttag_big)
-        if url != None:
-            data_buffer.insert(data_buffer.get_end_iter(), "\n")
-            data_buffer.insert_with_tags(data_buffer.get_end_iter(), url, texttag_url)
-            data_buffer.insert(data_buffer.get_end_iter(), "\n")
-        data_buffer.insert(data_buffer.get_end_iter(), contents)
-
-        self.ui.textview1.set_buffer(data_buffer)
+        return texttagtable
 
     def url_event_handler(self, _tag, _widget, event, _iter):
         # We also used to check event.button == 1 here, but event.button
@@ -667,7 +733,7 @@ class PasaffeWindow(Window):
 
         # TODO: make sure folder name is unique in same level
 
-        new_iter = self.ui.liststore1.append(treeiter, ['gtk-directory', 'New Folder',"pasaffe_treenode.New Folder"])
+        new_iter = self.ui.liststore1.append(treeiter, ['gtk-directory', _('New Folder'),"pasaffe_treenode.New Folder"])
         if treeiter != None:
             path = self.ui.treeview1.get_model().get_path(treeiter)
             self.ui.treeview1.expand_row(path, False)
@@ -718,10 +784,10 @@ class PasaffeWindow(Window):
             information = _('<big><b>Are you sure you wish to remove "%s"?</b></big>\n\n') % entry_name
             information += _('Contents of the entry will be lost.\n')
 
-            info_dialog = Gtk.MessageDialog(self,
-                                            Gtk.DialogFlags.MODAL,
-                                            Gtk.MessageType.QUESTION,
-                                            Gtk.ButtonsType.YES_NO)
+            info_dialog = Gtk.MessageDialog(transient_for=self,
+                                            modal=True,
+                                            message_type=Gtk.MessageType.QUESTION,
+                                            buttons=Gtk.ButtonsType.YES_NO)
             info_dialog.set_markup(information)
             result = info_dialog.run()
             info_dialog.destroy()
@@ -737,10 +803,10 @@ class PasaffeWindow(Window):
             information = _('<big><b>Are you sure you wish to remove folder "%s"?</b></big>\n\n') % folder_name
             information += _('All entries in this folder will be lost.\n')
 
-            info_dialog = Gtk.MessageDialog(self,
-                                            Gtk.DialogFlags.MODAL,
-                                            Gtk.MessageType.QUESTION,
-                                            Gtk.ButtonsType.YES_NO)
+            info_dialog = Gtk.MessageDialog(transient_for=self,
+                                            modal=True,
+                                            message_type=Gtk.MessageType.QUESTION,
+                                            buttons=Gtk.ButtonsType.YES_NO)
             info_dialog.set_markup(information)
             result = info_dialog.run()
             info_dialog.destroy()
@@ -795,7 +861,7 @@ class PasaffeWindow(Window):
             self.disable_idle_timeout()
             self.editdetails_dialog = self.EditDetailsDialog()
 
-            for record_type, widget_name in record_dict.items():
+            for record_type, widget_name in list(record_dict.items()):
                 # Handle folders separately
                 if record_type == 2:
                     liststore = self.editdetails_dialog.builder.get_object('liststore1')
@@ -814,7 +880,7 @@ class PasaffeWindow(Window):
             tree_changed = False
 
             if response == Gtk.ResponseType.OK:
-                for record_type, widget_name in record_dict.items():
+                for record_type, widget_name in list(record_dict.items()):
                     # Get the new value
                     if record_type == 2:
                         combo = self.editdetails_dialog.builder.get_object('folder_combo')
@@ -1179,10 +1245,10 @@ class PasaffeWindow(Window):
         if self.passfile.get_saved_application():
             information += _('Application used: %s\n') % self.passfile.get_saved_application()
 
-        info_dialog = Gtk.MessageDialog(self,
-                                        Gtk.DialogFlags.MODAL,
-                                        Gtk.MessageType.INFO,
-                                        Gtk.ButtonsType.OK)
+        info_dialog = Gtk.MessageDialog(transient_for=self,
+                                        modal=True,
+                                        message_type=Gtk.MessageType.INFO,
+                                        buttons=Gtk.ButtonsType.OK)
         info_dialog.set_markup(information)
         info_dialog.run()
         info_dialog.destroy()
