@@ -19,14 +19,14 @@ import os
 
 import gettext
 t = gettext.translation('pasaffe', fallback=True)
-_ = t.ugettext
+_ = t.gettext
 
 from gi.repository import Gio, Gtk  # pylint: disable=E0611
 
 from pasaffe import PasaffeWindow
 
 from pasaffe_lib import set_up_logging, get_version
-
+from pasaffe_lib.helpers import get_database_path
 
 def parse_options():
     """Support for command line options"""
@@ -46,26 +46,13 @@ def parse_options():
     return options
 
 
-def get_database_path():
-    """Determines standard XDG location for database"""
-    if 'XDG_DATA_HOME' in os.environ:
-        basedir = os.path.join(os.environ['XDG_DATA_HOME'], 'pasaffe')
-    else:
-        basedir = os.path.join(os.environ['HOME'], '.local/share/pasaffe')
-
-    if not os.path.exists(basedir):
-        os.mkdir(basedir, 0700)
-
-    return os.path.join(basedir, 'pasaffe.psafe3')
-
-
 def main():
     'constructor for your class instances'
     options = parse_options()
 
     filename = get_database_path()
 
-    settings = Gio.Settings("net.launchpad.pasaffe")
+    settings = Gio.Settings.new("net.launchpad.pasaffe")
 
     # On first launch, set the standard location
     if settings.get_string('database-path') == "":
