@@ -118,7 +118,7 @@ class Builder(Gtk.Builder):
         connection_dict = {}
         connection_dict.update(self.glade_handler_dict)
         connection_dict.update(callback_handler_dict)
-        for item in connection_dict.items():
+        for item in list(connection_dict.items()):
             if item[1] is None:
                 # the handler is missing so reroute to default_handler
                 handler = functools.partial(
@@ -165,7 +165,7 @@ class UiFactory():
     ''' provides an object with attributes as glade widgets'''
     def __init__(self, widget_dict):
         self._widget_dict = widget_dict
-        for (widget_name, widget) in widget_dict.items():
+        for (widget_name, widget) in list(widget_dict.items()):
             setattr(self, widget_name, widget)
 
         # Mangle any non-usable names (like with spaces or dashes)
@@ -175,7 +175,7 @@ class UiFactory():
         consider_message = """consider using a pythonic name instead""" + \
                            """ of design name '%s'"""
 
-        for (widget_name, widget) in widget_dict.items():
+        for (widget_name, widget) in list(widget_dict.items()):
             pyname = make_pyname(widget_name)
             if pyname != widget_name:
                 if hasattr(self, pyname):
@@ -186,7 +186,7 @@ class UiFactory():
 
         def iterator():
             '''Support 'for o in self' '''
-            return iter(widget_dict.values())
+            return iter(list(widget_dict.values()))
         setattr(self, '__iter__', iterator)
 
     def __getitem__(self, name):
@@ -259,7 +259,7 @@ def auto_connect_by_name(callback_obj, builder):
 
     callback_handler_dict = dict_from_callback_obj(callback_obj)
 
-    for item in builder.widgets.items():
+    for item in list(builder.widgets.items()):
         (widget_name, widget) = item
         signal_ids = []
         try:
@@ -294,7 +294,7 @@ def do_connect(item, signal_name, handler_names,
     widget_name, widget = item
 
     for handler_name in handler_names:
-        target = handler_name in callback_handler_dict.keys()
+        target = handler_name in list(callback_handler_dict.keys())
         connection = (widget_name, signal_name, handler_name)
         duplicate = connection in connections
         if target and not duplicate:
@@ -310,7 +310,7 @@ def log_unconnected_functions(callback_handler_dict, connections):
 
     connected_functions = [x[2] for x in connections]
 
-    handler_names = callback_handler_dict.keys()
+    handler_names = list(callback_handler_dict.keys())
     unconnected = [x for x in handler_names if x.startswith('on_')]
 
     for handler_name in connected_functions:
