@@ -62,7 +62,7 @@ class Window(Gtk.Window):
         self.preferences_dialog = None  # instance
         self.AboutDialog = None  # class
 
-        self.settings = Gio.Settings("net.launchpad.pasaffe")
+        self.settings = Gio.Settings.new("net.launchpad.pasaffe")
         self.settings.connect('changed', self.on_preferences_changed)
 
     def on_mnu_contents_activate(self, widget, data=None):
