@@ -116,7 +116,7 @@ class Blowfish:
     DECRYPT = 1
 
     # For the __round_func
-    modulus = long(2) ** 32
+    modulus = int(2) ** 32
 
     def __init__(self, key):
 
@@ -404,10 +404,10 @@ class Blowfish:
         key_len = len(key)
         index = 0
         for i in range(len(self.p_boxes)):
-            val = (ord(key[index % key_len]) << 24) + \
-                  (ord(key[(index + 1) % key_len]) << 16) + \
-                  (ord(key[(index + 2) % key_len]) << 8) + \
-                   ord(key[(index + 3) % key_len])
+            val = (key[index % key_len] << 24) + \
+                  (key[(index + 1) % key_len] << 16) + \
+                  (key[(index + 2) % key_len] << 8) + \
+                   key[(index + 3) % key_len]
             self.p_boxes[i] = self.p_boxes[i] ^ val
             index = index + 4
 
@@ -457,10 +457,10 @@ class Blowfish:
 
         # Perform all ops as longs then and out the last 32-bits to
         # obtain the integer
-        f = (long(self.s_boxes[0][a]) + \
-             long(self.s_boxes[1][b])) % self.modulus
-        f = f ^ long(self.s_boxes[2][c])
-        f = f + long(self.s_boxes[3][d])
+        f = (int(self.s_boxes[0][a]) + \
+             int(self.s_boxes[1][b])) % self.modulus
+        f = f ^ int(self.s_boxes[2][c])
+        f = f + int(self.s_boxes[3][d])
         f = (f % self.modulus) & 0xFFFFFFFF
 
         return f
@@ -471,19 +471,22 @@ class Blowfish:
                                " length: %s" % len(data))
 
         # Use big endianess since that's what everyone else uses
-        xl = ord(data[3]) | (ord(data[2]) << 8) | (ord(data[1]) << 16) | \
-            (ord(data[0]) << 24)
-        xr = ord(data[7]) | (ord(data[6]) << 8) | (ord(data[5]) << 16) | \
-            (ord(data[4]) << 24)
+        xl = data[3] | (data[2] << 8) | (data[1] << 16) | \
+            (data[0] << 24)
+        xr = data[7] | (data[6] << 8) | (data[5] << 16) | \
+            (data[4] << 24)
 
         cl, cr = self.cipher(xl, xr, self.ENCRYPT)
-        chars = ''.join([
-            chr((cl >> 24) & 0xFF), chr((cl >> 16) & 0xFF),
-            chr((cl >> 8) & 0xFF), chr(cl & 0xFF),
-            chr((cr >> 24) & 0xFF), chr((cr >> 16) & 0xFF),
-            chr((cr >> 8) & 0xFF), chr(cr & 0xFF)
-        ])
-        return chars
+        chars = bytearray()
+        chars.append((cl >> 24) & 0xFF)
+        chars.append((cl >> 16) & 0xFF)
+        chars.append((cl >> 8) & 0xFF)
+        chars.append(cl & 0xFF)
+        chars.append((cr >> 24) & 0xFF)
+        chars.append((cr >> 16) & 0xFF)
+        chars.append((cr >> 8) & 0xFF)
+        chars.append(cr & 0xFF)
+        return bytes(chars)
 
     def decrypt(self, data):
         if not len(data) == 8:
@@ -491,19 +494,22 @@ class Blowfish:
                                " length: %s" % len(data))
 
         # Use big endianess since that's what everyone else uses
-        cl = ord(data[3]) | (ord(data[2]) << 8) | \
-            (ord(data[1]) << 16) | (ord(data[0]) << 24)
-        cr = ord(data[7]) | (ord(data[6]) << 8) | \
-            (ord(data[5]) << 16) | (ord(data[4]) << 24)
+        cl = data[3] | (data[2] << 8) | \
+            (data[1] << 16) | (data[0] << 24)
+        cr = data[7] | (data[6] << 8) | \
+            (data[5] << 16) | (data[4] << 24)
 
         xl, xr = self.cipher(cl, cr, self.DECRYPT)
-        chars = ''.join([
-            chr((xl >> 24) & 0xFF), chr((xl >> 16) & 0xFF),
-            chr((xl >> 8) & 0xFF), chr(xl & 0xFF),
-            chr((xr >> 24) & 0xFF), chr((xr >> 16) & 0xFF),
-            chr((xr >> 8) & 0xFF), chr(xr & 0xFF)
-        ])
-        return chars
+        chars = bytearray()
+        chars.append((xl >> 24) & 0xFF)
+        chars.append((xl >> 16) & 0xFF)
+        chars.append((xl >> 8) & 0xFF)
+        chars.append(xl & 0xFF)
+        chars.append((xr >> 24) & 0xFF)
+        chars.append((xr >> 16) & 0xFF)
+        chars.append((xr >> 8) & 0xFF)
+        chars.append(xr & 0xFF)
+        return bytes(chars)
 
     # ==== CBC Mode ====
     def initCBC(self, iv=0):
@@ -518,39 +524,39 @@ class Blowfish:
         with this method one after the other without any intermediate work.
         Each buffer must be a multiple of 8-octets (64-bits) in length.
         """
-        if type(data) != types.StringType:
+        if type(data) != bytes:
             raise RuntimeError("Can only work on 8-bit strings")
         if (len(data) % 8) != 0:
             raise RuntimeError("Can only work with data in 64-bit"
                                " multiples in CBC mode")
 
-        xor = lambda t: ord(t[0]) ^ ord(t[1])
-        result = ''
+        xor = lambda t: t[0] ^ t[1]
+        result = b''
         block_size = self.block_size()
         for i in range(0, len(data), block_size):
             p_block = data[i:i + block_size]
-            pair = zip(p_block, self.cbc_iv)
-            j_block = ''.join(map(chr, map(xor, pair)))
+            pair = list(zip(p_block, self.cbc_iv))
+            j_block = bytes(list(map(xor, pair)))
             c_block = self.encrypt(j_block)
             result += c_block
             self.cbc_iv = c_block
         return result
 
     def decryptCBC(self, data):
-        if type(data) != types.StringType:
+        if type(data) != bytes:
             raise RuntimeError("Can only work on 8-bit strings")
         if (len(data) % 8) != 0:
             raise RuntimeError("Can only work with data in 64-bit"
                                " multiples in CBC mode")
 
-        xor = lambda t: ord(t[0]) ^ ord(t[1])
-        result = ''
+        xor = lambda t: t[0] ^ t[1]
+        result = b''
         block_size = self.block_size()
         for i in range(0, len(data), block_size):
             c_block = data[i:i + block_size]
             j_block = self.decrypt(c_block)
-            pair = zip(j_block, self.cbc_iv)
-            p_block = ''.join(map(chr, map(xor, pair)))
+            pair = list(zip(j_block, self.cbc_iv))
+            p_block = bytes(list(map(xor, pair)))
             result += p_block
             self.cbc_iv = c_block
         return result
@@ -571,7 +577,7 @@ class Blowfish:
 
     def _nextCTRByte(self):
         """Returns one byte of CTR keystream"""
-        b = ord(self.ctr_cks[self.ctr_pos])
+        b = self.ctr_cks[self.ctr_pos]
         self.ctr_pos += 1
         if self.ctr_pos >= len(self.ctr_cks):
             self._calcCTRBUF()
@@ -583,12 +589,12 @@ class Blowfish:
         (belonging to the same logical stream of buffers) can be encrypted
         with this method one after the other without any intermediate work.
         """
-        if type(data) != types.StringType:
+        if type(data) != bytes:
             raise RuntimeException("Can only work on 8-bit strings")
-        result = []
+        result = bytearray()
         for ch in data:
-            result.append(chr(ord(ch) ^ self._nextCTRByte()))
-        return "".join(result)
+            result.append(ch ^ self._nextCTRByte())
+        return bytes(result)
 
     def decryptCTR(self, data):
         return self.encryptCTR(data)
@@ -607,20 +613,20 @@ class Blowfish:
         import binascii
         # for more vectors see http://www.schneier.com/code/vectors.txt
         vectors = (
-            ('0000000000000000', '0000000000000000', '4EF997456198DD78'),
-            ('FFFFFFFFFFFFFFFF', 'FFFFFFFFFFFFFFFF', '51866FD5B85ECB8A'),
-            ('3000000000000000', '1000000000000001', '7D856F9A613063F2'),
-            ('1111111111111111', '1111111111111111', '2466DD878B963C9D'),
-            ('49E95D6D4CA229BF', '02FE55778117F12A', 'CF9C5D7A4986ADB5'),
-            ('E0FEE0FEF1FEF1FE', '0123456789ABCDEF', 'C39E072D9FAC631D'),
-            ('07A7137045DA2A16', '3BDD119049372802', '2EEDDA93FFD39C79'),
+            (b'0000000000000000', b'0000000000000000', b'4EF997456198DD78'),
+            (b'FFFFFFFFFFFFFFFF', b'FFFFFFFFFFFFFFFF', b'51866FD5B85ECB8A'),
+            (b'3000000000000000', b'1000000000000001', b'7D856F9A613063F2'),
+            (b'1111111111111111', b'1111111111111111', b'2466DD878B963C9D'),
+            (b'49E95D6D4CA229BF', b'02FE55778117F12A', b'CF9C5D7A4986ADB5'),
+            (b'E0FEE0FEF1FEF1FE', b'0123456789ABCDEF', b'C39E072D9FAC631D'),
+            (b'07A7137045DA2A16', b'3BDD119049372802', b'2EEDDA93FFD39C79'),
         )
         ok = True
         for v in vectors:
             c = Blowfish(binascii.a2b_hex(v[0]))
             e = binascii.b2a_hex(c.encrypt(binascii.a2b_hex(v[1]))).upper()
             if e != v[2]:
-                print "VECTOR TEST FAIL: expecting %s, got %s" % (repr(v), e)
+                print("VECTOR TEST FAIL: expecting %s, got %s" % (repr(v), e))
                 ok = False
         return ok
 
@@ -629,65 +635,66 @@ class Blowfish:
 
 if __name__ == '__main__':
     if not Blowfish.testVectors():
-        print "WARNING: The implementation doesn't pass " + \
-              "algorithm test vectors!"
+        print("WARNING: The implementation doesn't pass " + \
+              "algorithm test vectors!")
     else:
-        print "The implementation passes algorithm test vectors (ECB)."
+        print("The implementation passes algorithm test vectors (ECB).")
 
-    key = 'This is a test key'
+    key = b'This is a test key'
     cipher = Blowfish(key)
 
-    print "Testing encryption:"
+    print("Testing encryption:")
     xl = 123456
     xr = 654321
-    print "\tPlain text: (%s, %s)" % (xl, xr)
+    print("\tPlain text: (%s, %s)" % (xl, xr))
     cl, cr = cipher.cipher(xl, xr, cipher.ENCRYPT)
-    print "\tCrypted is: (%s, %s)" % (cl, cr)
+    print("\tCrypted is: (%s, %s)" % (cl, cr))
     dl, dr = cipher.cipher(cl, cr, cipher.DECRYPT)
-    print "\tUnencrypted is: (%s, %s)" % (dl, dr)
+    print("\tUnencrypted is: (%s, %s)" % (dl, dr))
 
-    print "Testing block encrypt:"
-    text = 'testtest'
-    print "\tText:\t\t%s" % text
+    print("Testing block encrypt:")
+    text = b'testtest'
+    print("\tText:\t\t%s" % text)
     crypted = cipher.encrypt(text)
-    print "\tEncrypted:\t%s" % repr(crypted)
+    print("\tEncrypted:\t%s" % repr(crypted))
     decrypted = cipher.decrypt(crypted)
-    print "\tDecrypted:\t%s" % decrypted
+    print("\tDecrypted:\t%s" % decrypted)
 
-    print "Testing CTR encrypt:"
+    print("Testing CTR encrypt:")
     cipher.initCTR()
-    text = "The quick brown fox jumps over the lazy dog"
-    print "\tText:\t\t", text
+    text = b"The quick brown fox jumps over the lazy dog"
+    print("\tText:\t\t", text)
     crypted = cipher.encryptCTR(text)
-    print "\tEncrypted:\t", repr(crypted)
+    print("\tEncrypted:\t", repr(crypted))
     cipher.initCTR()
     decrypted = cipher.decryptCTR(crypted)
-    print "\tDecrypted:\t", decrypted
+    print("\tDecrypted:\t", decrypted)
 
-    print "Testing CBC encrypt:"
+    print("Testing CBC encrypt:")
     cipher.initCBC()
-    text = "Owen's Ornery Old Oryx Obstructed Olga's Optics."
-    print "\tText:\t\t", text
+    text = b"Owen's Ornery Old Oryx Obstructed Olga's Optics."
+    print("\tText:\t\t", text)
     crypted = cipher.encryptCBC(text)
-    print "\tEncrypted:\t", repr(crypted)
+    print("\tEncrypted:\t", repr(crypted))
     cipher.initCBC()
     decrypted = cipher.decryptCBC(crypted)
-    print "\tDecrypted:\t", decrypted
+    print("\tDecrypted:\t", decrypted)
 
-    print "Testing speed"
+    print("Testing speed")
     from time import time
     t1 = time()
     n = 0
     tlen = 0
     while True:
-        for i in xrange(1000):
+        for i in range(1000):
             tstr = "The quick brown fox jumps over the lazy dog %d" % i
-            enc = cipher.encryptCTR(tstr)
-            tlen += len(tstr)
+            tbts = tstr.encode('utf-8')
+            enc = cipher.encryptCTR(tbts)
+            tlen += len(tbts)
         n += 1000
         t2 = time()
         if t2 - t1 > 5:
             break
     t = t2 - t1
-    print "%d encryptions in %0.1f seconds:" + \
-          " %0.1f enc/s, %0.1f bytes/s" % (n, t, n / t, tlen / t)
+    print("%d encryptions in %0.1f seconds:" % (n, t) +
+          " %0.1f enc/s, %0.1f bytes/s" % (n / t, tlen / t))
