@@ -30,6 +30,8 @@ __all__ = [
 __pasaffe_data_directory__ = '../data/'
 __license__ = 'GPL-3'
 __version__ = '0'
+__help_prefix__ = 'help:'
+__help_separator__ = '/'
 
 import os
 
@@ -73,3 +75,11 @@ def get_data_path():
 
 def get_version():
     return __version__
+
+
+def get_help_prefix():
+    return __help_prefix__
+
+
+def get_help_separator():
+    return __help_separator__
